@@ -523,6 +523,23 @@ rsvg_xinclude_handler_end (RsvgSaxHandler * self, const char *name)
     }
 }
 
+static void
+_rsvg_set_xml_parse_options(xmlParserCtxtPtr xml_parser,
+                            RsvgHandle *ctx)
+{
+    xml_parser->options |= XML_PARSE_NONET;
+
+    if (ctx->priv->flags & RSVG_HANDLE_FLAG_UNLIMITED) {
+#if LIBXML_VERSION > 20632
+        xml_parser->options |= XML_PARSE_HUGE;
+#endif
+    }
+
+#if LIBXML_VERSION > 20800
+    xml_parser->options |= XML_PARSE_BIG_LINES;
+#endif
+}
+
 /* http://www.w3.org/TR/xinclude/ */
 static void
 rsvg_start_xinclude (RsvgHandle * ctx, RsvgPropertyBag * atts)
@@ -575,7 +592,7 @@ rsvg_start_xinclude (RsvgHandle * ctx, RsvgPropertyBag * atts)
             goto fallback;
 
         xml_parser = xmlCreatePushParserCtxt (&rsvgSAXHandlerStruct, ctx, NULL, 0, NULL);
-        xml_parser->options |= XML_PARSE_NONET;
+        _rsvg_set_xml_parse_options(xml_parser, ctx);
 
         buffer = _rsvg_xml_input_buffer_new_from_stream (stream, NULL /* cancellable */, XML_CHAR_ENCODING_NONE, &err);
         g_object_unref (stream);
@@ -1115,7 +1132,7 @@ rsvg_handle_write_impl (RsvgHandle * handle, const guchar * buf, gsize count, GE
     if (handle->priv->ctxt == NULL) {
         handle->priv->ctxt = xmlCreatePushParserCtxt (&rsvgSAXHandlerStruct, handle, NULL, 0,
                                                       rsvg_handle_get_base_uri (handle));
-        handle->priv->ctxt->options |= XML_PARSE_NONET;
+        _rsvg_set_xml_parse_options(handle->priv->ctxt, handle);
 
         /* if false, external entities work, but internal ones don't. if true, internal entities
            work, but external ones don't. favor internal entities, in order to not cause a
@@ -1204,9 +1221,9 @@ rsvg_drawing_ctx_free (RsvgDrawingCtx * handle)
  * @handle: An #RsvgHandle
  *
  * Returns the SVG's metadata in UTF-8 or %NULL. You must make a copy
- * of this metadata if you wish to use it after #handle has been freed.
+ * of this metadata if you wish to use it after @handle has been freed.
  *
- * Returns: The SVG's title
+ * Returns: (nullable): The SVG's title
  *
  * Since: 2.9
  *
@@ -1228,9 +1245,9 @@ rsvg_handle_get_metadata (RsvgHandle * handle)
  * @handle: An #RsvgHandle
  *
  * Returns the SVG's title in UTF-8 or %NULL. You must make a copy
- * of this title if you wish to use it after #handle has been freed.
+ * of this title if you wish to use it after @handle has been freed.
  *
- * Returns: The SVG's title
+ * Returns: (nullable): The SVG's title
  *
  * Since: 2.4
  *
@@ -1252,9 +1269,9 @@ rsvg_handle_get_title (RsvgHandle * handle)
  * @handle: An #RsvgHandle
  *
  * Returns the SVG's description in UTF-8 or %NULL. You must make a copy
- * of this description if you wish to use it after #handle has been freed.
+ * of this description if you wish to use it after @handle has been freed.
  *
- * Returns: The SVG's description
+ * Returns: (nullable): The SVG's description
  *
  * Since: 2.4
  *
@@ -1301,9 +1318,10 @@ rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_d
  * rsvg_handle_get_dimensions_sub:
  * @handle: A #RsvgHandle
  * @dimension_data: (out): A place to store the SVG's size
- * @id: An element's id within the SVG, or NULL to get the dimension of the whole SVG. 
- * For example, if you have a layer called "layer1" for that you want to get the dimension, 
- * pass "#layer1" as the id.
+ * @id: (nullable): An element's id within the SVG, or %NULL to get
+ *   the dimension of the whole SVG.  For example, if you have a layer
+ *   called "layer1" for that you want to get the dimension, pass
+ *   "#layer1" as the id.
  *
  * Get the size of a subelement of the SVG file. Do not call from within the size_func callback, because an infinite loop will occur.
  *
@@ -1530,7 +1548,7 @@ rsvg_handle_has_sub (RsvgHandle * handle,
  * @dpi: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the all future outgoing pixbufs. Common values are
- * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to @dpi will
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.8
@@ -1547,7 +1565,7 @@ rsvg_set_default_dpi (double dpi)
  * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the all future outgoing pixbufs. Common values are
- * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to @dpi will
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.8
@@ -1572,7 +1590,7 @@ rsvg_set_default_dpi_x_y (double dpi_x, double dpi_y)
  * @dpi: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the outgoing pixbuf. Common values are
- * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to @dpi will
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.8
@@ -1590,7 +1608,7 @@ rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
  * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the outgoing pixbuf. Common values are
- * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi_x or #dpi_y will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi_x or @dpi_y will
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.8
@@ -1614,7 +1632,7 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
 /**
  * rsvg_handle_set_size_callback:
  * @handle: An #RsvgHandle
- * @size_func: A sizing function, or %NULL
+ * @size_func: (nullable): A sizing function, or %NULL
  * @user_data: User data to pass to @size_func, or %NULL
  * @user_data_destroy: Destroy function for @user_data, or %NULL
  *
@@ -1648,9 +1666,9 @@ rsvg_handle_set_size_callback (RsvgHandle * handle,
  * @count: length of the @buf buffer in bytes
  * @error: (allow-none): a location to store a #GError, or %NULL
  *
- * Loads the next @count bytes of the image.  This will return #TRUE if the data
- * was loaded successful, and #FALSE if an error occurred.  In the latter case,
- * the loader will be closed, and will not accept further writes. If FALSE is
+ * Loads the next @count bytes of the image.  This will return %TRUE if the data
+ * was loaded successful, and %FALSE if an error occurred.  In the latter case,
+ * the loader will be closed, and will not accept further writes. If %FALSE is
  * returned, @error will be set to an error from the #RsvgError domain. Errors
  * from #GIOErrorEnum are also possible.
  *
@@ -1738,7 +1756,7 @@ rsvg_handle_close (RsvgHandle * handle, GError ** error)
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by
  * triggering the cancellable object from another thread. If the
- * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * operation was cancelled, the error %G_IO_ERROR_CANCELLED will be
  * returned.
  *
  * Returns: %TRUE if reading @stream succeeded, or %FALSE otherwise
@@ -1772,7 +1790,7 @@ rsvg_handle_read_stream_sync (RsvgHandle   *handle,
     if (priv->ctxt == NULL) {
         priv->ctxt = xmlCreatePushParserCtxt (&rsvgSAXHandlerStruct, handle, NULL, 0,
                                               rsvg_handle_get_base_uri (handle));
-        priv->ctxt->options |= XML_PARSE_NONET;
+        _rsvg_set_xml_parse_options(priv->ctxt, handle);
 
         /* if false, external entities work, but internal ones don't. if true, internal entities
            work, but external ones don't. favor internal entities, in order to not cause a
@@ -1835,7 +1853,7 @@ rsvg_handle_read_stream_sync (RsvgHandle   *handle,
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by
  * triggering the cancellable object from another thread. If the
- * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * operation was cancelled, the error %G_IO_ERROR_CANCELLED will be
  * returned.
  *
  * Returns: a new #RsvgHandle on success, or %NULL with @error filled in
@@ -1878,7 +1896,7 @@ rsvg_handle_new_from_gfile_sync (GFile          *file,
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by
  * triggering the cancellable object from another thread. If the
- * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * operation was cancelled, the error %G_IO_ERROR_CANCELLED will be
  * returned.
  *
  * Returns: a new #RsvgHandle on success, or %NULL with @error filled in
